// Copyright (C) 2013-2015 Altera Corporation, San Jose, California, USA. All rights reserved.
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to
// whom the Software is furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// 
// This agreement shall be governed in all respects by the laws of the State of California and
// by the laws of the United States of America.

// include coefficients
#include "mnist_coef_100.h"

 // ACL kernel for adding two input vectors
__kernel void mnist_aocl(__global const float *x, 
                         __global const float *y, 
                         __global int *restrict z)
{
    // get index of the work item
    int index = get_global_id(0);

	// buffer memories
    float f2[100][1][1];
    float f3[100][1][1];    
    float f4[10][1][1];

    int i, j, k;

    // loop control variables
    int stride, ksiz, n_ofeat, n_infeat, infeat_siz;
    int ofeat, oy, inf_y, ox, inf_x, infeat, ky, kx;
    float tmp;

    stride = 1;
    ksiz   = 1;

	//printf("index=%d\n", index);

    // layer 1 (fully connection layer) -------------------------
    n_ofeat = 100;
    n_infeat = 1;
    infeat_siz = 28;
    for( ofeat = 0; ofeat < n_ofeat; ofeat++){
        oy = 0;
        tmp = 0.0;
        for( infeat = 0; infeat < n_infeat; infeat++){
            for( inf_y = 0; inf_y < infeat_siz; inf_y++){
                for( inf_x = 0; inf_x < infeat_siz; inf_x++){
					tmp = tmp + (x[(index*28*28)+inf_y*28+inf_x] * l1w[ofeat][oy]);
                    oy = oy + 1;
                }
				
            }
        }

        tmp = tmp + l1b[ofeat];
        if( tmp > 0.0)
            f2[ofeat][0][0] = tmp;
        else
            f2[ofeat][0][0] = 0.0;
    }

    // layer 2 (fully connection layer) -------------------------
    n_ofeat = 100;
    n_infeat = 100;
    infeat_siz = 1;
    for( ofeat = 0; ofeat < n_ofeat; ofeat++){
        oy = 0;
        tmp = 0.0;
        for( infeat = 0; infeat < n_infeat; infeat++){
            for( inf_y = 0; inf_y < infeat_siz; inf_y++){
                for( inf_x = 0; inf_x < infeat_siz; inf_x++){
                    tmp = tmp + (f2[infeat][inf_y][inf_x] * l2w[ofeat][oy]);
                    oy = oy + 1;
                }
            }
        }

        tmp = tmp + l2b[ofeat];
        if( tmp > 0.0)
            f3[ofeat][0][0] = tmp;
        else
            f3[ofeat][0][0] = 0.0;
    }

    // layer 3 (fully connection layer) -------------------------
    n_ofeat = 10;
    n_infeat = 100;
    infeat_siz = 1;
    for( ofeat = 0; ofeat < n_ofeat; ofeat++){
        oy = 0;
        tmp = 0.0;
        for( infeat = 0; infeat < n_infeat; infeat++){
            for( inf_y = 0; inf_y < infeat_siz; inf_y++){
                for( inf_x = 0; inf_x < infeat_siz; inf_x++){
                    tmp = tmp + (f3[infeat][inf_y][inf_x] * l3w[ofeat][oy]);
                    oy = oy + 1;
                }
            }
        }

        tmp = tmp + l3b[ofeat];
        f4[ofeat][0][0] = tmp;
    }

    // decision the next movement
    int loop;
    float max_val;
    int max_idx;

    max_idx = 0;
    max_val = -9999.9;

    for( loop = 0; loop < 10; loop++){
        if( f4[loop][0][0] > max_val){
            max_val = f4[loop][0][0];
            max_idx = loop;
        }
    }

	z[index] = max_idx | (index << 3);
}

